package org.eclipse.albireo.examples.plugin.views;

import java.awt.Dimension;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.part.ViewPart;

/**
 * This view arranges a 5 x 4 grid of controls in an SWT GridLayout.
 * The elements in the grid can be either Swing controls or SWT controls.
 * You can switch back and forth between Swing and SWT to see the differences
 * in layout.
 */
public class GridLayoutView extends ViewPart {

    public static String ID="org.eclipse.albireo.examples.plugin.gridLayoutView"; //$NON-NLS-1$
    private Composite mainComposite;
    protected boolean isSwing = true;
    private Control grid;

    public GridLayoutView() {
    }
    
    // Embedding a Swing button is not really a good idea in a real application. This is 
    // just a simple way to demonstrate/test swing components participating in a grid layout
    // A real application would also combine all of these swing components into a single
    // embedded SwingControl. 
    
    static class SwingButton extends SwingControl {
        
        private final String text;

        SwingButton(Composite parent, String text) {
            super(parent, SWT.NONE);
            this.text = text;
        }

        protected JComponent createSwingComponent() {
            return new JButton(text);
        }

        public Composite getLayoutAncestor() {
            return getParent();
        }
    }

    static class SwingTable extends SwingControl {
        
        private final int rowCount;
        private final int colCount;

        SwingTable(Composite parent, int rowCount, int colCount) {
            super(parent, SWT.NONE);
            this.rowCount = rowCount;
            this.colCount = colCount;
        }

        protected JComponent createSwingComponent() {
            /* Creating components */
            Vector rows = new Vector();
            for (int i = 0; i < rowCount; i++) {
                Vector row = new Vector();
                for (int j = 0; j < colCount; j++) {
                    row.addElement("Item " + i + "-" + j);
                }
                rows.addElement(row);
            }
            Vector columns = new Vector();
            for (int i = 0; i < colCount; i++) {
                columns.addElement("Column " + i);
            }
            JTable table = new JTable(rows, columns);
            table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
            table.createDefaultColumnsFromModel();
            
            JScrollPane scrollPane = new JScrollPane(table);
//            scrollPane.setBorder(new EmptyBorder(0,0,0,0));
            // Reduce the default height of 403 pixels.
            scrollPane.setPreferredSize(
                new Dimension(scrollPane.getPreferredSize().width,
                              Math.min(scrollPane.getPreferredSize().height,200)));
            
            return scrollPane;
        }

        public Composite getLayoutAncestor() {
            return getParent();
        }
    }
    
    public void createPartControl(Composite parent) {
        mainComposite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        mainComposite.setLayout(layout);
        
        Control panel = createControlPanel(mainComposite);
        GridData data = new GridData();
        data.horizontalAlignment = SWT.FILL;
        data.verticalAlignment = SWT.FILL;
        data.grabExcessHorizontalSpace = true;
        panel.setLayoutData(data);
        
        Control separator = new Label(mainComposite, SWT.SEPARATOR | SWT.HORIZONTAL);
        data = new GridData();
        data.horizontalAlignment = SWT.FILL;
        data.verticalAlignment = SWT.CENTER;
        data.grabExcessHorizontalSpace = true;
        separator.setLayoutData(data);
        
        grid = createGrid(mainComposite);
    }

    private Control createControlPanel(Composite parent) {
        
        Composite composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 2;
        composite.setLayout(layout);
        
        Group group = new Group(composite, SWT.NONE);
        layout = new GridLayout();
        layout.numColumns = 1;
        group.setLayout(layout);
        GridData data = new GridData();
        data.horizontalAlignment = SWT.FILL;
        data.verticalAlignment = SWT.FILL;
        group.setLayoutData(data);
        
        
        Button[] radios = new Button[2];

        radios[0] = new Button(group, SWT.RADIO);
        radios[0].setSelection(true);
        radios[0].setText("Swing");
        radios[0].addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }
            public void widgetSelected(SelectionEvent e) {
                isSwing = true;
                updateGrid();
            }
        });
        data = new GridData();
        data.horizontalAlignment = SWT.LEFT;
        data.verticalAlignment = SWT.CENTER;
        radios[0].setLayoutData(data);
        

        radios[1] = new Button(group, SWT.RADIO);
        radios[1].setText("SWT");
        radios[1].addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }
            public void widgetSelected(SelectionEvent e) {
                isSwing = false;
                updateGrid();
            }
        });
        data = new GridData();
        data.horizontalAlignment = SWT.LEFT;
        data.verticalAlignment = SWT.CENTER;
        radios[0].setLayoutData(data);

        return composite;

    }

    protected void updateGrid() {
        grid.dispose();
        grid = createGrid(mainComposite);
        mainComposite.layout();
    }

    private Control createGrid(Composite parent) {
        Composite composite = new Composite(parent, SWT.NONE);
        GridData data =  new GridData();
        data.horizontalAlignment = SWT.FILL;
        data.verticalAlignment = SWT.FILL;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = true;
        composite.setLayoutData(data);
        
        GridLayout layout = new GridLayout(4, false);
        composite.setLayout(layout);
        
        Control c = createTableControl(composite, 4, 4);
        c.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true, 1, 1));
        c = createTableControl(composite, 4, 4);
        c.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
        c = createTableControl(composite, 4, 4);
        c.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, true, 1, 1));
        c = createTableControl(composite, 4, 4);
        c.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        
        c = createButtonControl(composite, "LEFT, TOP");
        c.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, true, 1, 1));
        c = createButtonControl(composite, "LEFT, CENTER");
        c.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true, 1, 1));
        c = createButtonControl(composite, "LEFT, BOTTOM");
        c.setLayoutData(new GridData(SWT.LEFT, SWT.BOTTOM, true, true, 1, 1));
        c = createButtonControl(composite, "LEFT, FILL");
        c.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, true, true, 1, 1));
        
        c = createButtonControl(composite, "CENTER, TOP");
        c.setLayoutData(new GridData(SWT.CENTER, SWT.TOP, true, true, 1, 1));
        c = createButtonControl(composite, "CENTER, CENTER");
        c.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true, 1, 1));
        c = createButtonControl(composite, "CENTER, BOTTOM");
        c.setLayoutData(new GridData(SWT.CENTER, SWT.BOTTOM, true, true, 1, 1));
        c = createButtonControl(composite, "CENTER, FILL");
        c.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, true, true, 1, 1));
        
        c = createButtonControl(composite, "RIGHT, TOP");
        c.setLayoutData(new GridData(SWT.RIGHT, SWT.TOP, true, true, 1, 1));
        c = createButtonControl(composite, "RIGHT, CENTER");
        c.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, true, 1, 1));
        c = createButtonControl(composite, "RIGHT, BOTTOM");
        c.setLayoutData(new GridData(SWT.RIGHT, SWT.BOTTOM, true, true, 1, 1));
        c = createButtonControl(composite, "RIGHT, FILL");
        c.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, true, true, 1, 1));
        
        c = createButtonControl(composite, "FILL, TOP");
        c.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true, 1, 1));
        c = createButtonControl(composite, "FILL, CENTER");
        c.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
        c = createButtonControl(composite, "FILL, BOTTOM");
        c.setLayoutData(new GridData(SWT.FILL, SWT.BOTTOM, true, true, 1, 1));
        c = createButtonControl(composite, "FILL, FILL");
        c.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        return composite;
    }

    private Control createButtonControl(Composite composite, String string) {
        if (isSwing) {
            return new SwingButton(composite, string);
        } else {
            Button b = new Button(composite, SWT.PUSH);
            b.setText(string);
            return b;
        }
    }

    private Control createTableControl(Composite composite, int rowCount, int colCount) {
        if (isSwing) {
            return new SwingTable(composite, rowCount, colCount);
        } else {
            return createSWTTable(composite, rowCount, colCount);
        }
    }

    private Control createSWTTable(Composite composite, int rowCount, int colCount) {
        Table table = new Table (composite, SWT.MULTI | SWT.BORDER | SWT.FULL_SELECTION);
        table.setLinesVisible (true);
        table.setHeaderVisible (true);
        for (int i=0; i<colCount; i++) {
            TableColumn column = new TableColumn (table, SWT.NONE);
            column.setText ("Column " + i);
        }   
        for (int i=0; i<rowCount; i++) {
            TableItem item = new TableItem (table, SWT.NONE);
            for (int j=0; j<colCount; j++) {
                item.setText(j, "Item " + i + "-" + j);
            }
        }
        for (int i=0; i<colCount; i++) {
            table.getColumn (i).pack ();
        }  
        return table;
    }

    public void setFocus() {
        mainComposite.setFocus();
    }

}
