package org.eclipse.albireo.examples.plugin.views;


import java.awt.EventQueue;

import javax.swing.SwingUtilities;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.part.ViewPart;


/**
 * This view demonstrates how a Swing control whose contents increase in size can
 * successfully participate in a SWT layout. As in the case SWT controls, it is necessary to
 * explicitly re-layout the containing composite.  
 */
public class RelayoutExampleView extends ViewPart {
    private SwingGrowShrinkControl leftSwingControl;
    private SwingGrowShrinkControl rightSwingControl;
    private int numCreatedControls = 0;

    private Composite composite;
    private Button shrinkSWTButton;

    public void createPartControl(Composite parent) {
        composite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        composite.setLayout(layout);
        
        Composite topComposite = new Composite(composite, SWT.NONE);
        GridData data = new GridData();
        data.horizontalAlignment = SWT.CENTER;
        data.verticalAlignment = SWT.CENTER;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = true;
        topComposite.setLayoutData(data);
        topComposite.setLayout(new GridLayout(2, false));  // single column grid
        
        leftSwingControl =
            new SwingGrowShrinkControl(topComposite, SWT.NONE) {
                public Composite getLayoutAncestor() {
                    return composite;
                }
                protected void growOrShrink(boolean grow) {
                    super.growOrShrink(grow);
                    updateShrinkSWTButton();
                }
                protected void afterComponentCreatedAWTThread() {
                    someControlCreated();
                }
            };
        data = new GridData();
        data.horizontalAlignment = SWT.CENTER;
        data.verticalAlignment = SWT.CENTER;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = true;
        leftSwingControl.setLayoutData(data);

        rightSwingControl =
            new SwingGrowShrinkControl(topComposite, SWT.NONE) {
                public Composite getLayoutAncestor() {
                    return composite;
                }
                protected void growOrShrink(boolean grow) {
                    super.growOrShrink(grow);
                    updateShrinkSWTButton();
                }
                protected void afterComponentCreatedAWTThread() {
                    someControlCreated();
                }
            };
        data = new GridData();
        data.horizontalAlignment = SWT.CENTER;
        data.verticalAlignment = SWT.CENTER;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = true;
        rightSwingControl.setLayoutData(data);

        Composite buttonList = new Composite(composite, SWT.NONE);
        data = new GridData();
        data.horizontalAlignment = SWT.CENTER;
        data.verticalAlignment = SWT.CENTER;
        data.grabExcessHorizontalSpace = true;
        data.horizontalSpan = 2;
        buttonList.setLayoutData(data);
        GridLayout buttonLayout = new GridLayout();
        buttonLayout.numColumns = 1;
        buttonList.setLayout(buttonLayout);

        Button growSWTButton = new Button(buttonList, SWT.PUSH);
        growSWTButton.setText("Grow Both (SWT Button)");
        data = new GridData();
        data.horizontalAlignment = SWT.LEAD;
        data.verticalAlignment = SWT.CENTER;
        growSWTButton.setLayoutData(data);

        shrinkSWTButton = new Button(buttonList, SWT.PUSH);
        shrinkSWTButton.setText("Shrink Both (SWT Button)");
        data = new GridData();
        data.horizontalAlignment = SWT.LEAD;
        data.verticalAlignment = SWT.CENTER;
        shrinkSWTButton.setLayoutData(data);

        Button layoutSWTButton = new Button(buttonList, SWT.PUSH);
        layoutSWTButton.setText("SWT-Layout (SWT Button)");
        data = new GridData();
        data.horizontalAlignment = SWT.LEAD;
        data.verticalAlignment = SWT.CENTER;
        layoutSWTButton.setLayoutData(data);

        growSWTButton.addSelectionListener(
            new SelectionListener() {
                public void widgetSelected(SelectionEvent e) {
                    SwingUtilities.invokeLater(
                        new Runnable() {
                            public void run() {
                                rightSwingControl.growOrShrink(true);
                                leftSwingControl.growOrShrink(true);
                            }
                        });
                }
                public void widgetDefaultSelected(SelectionEvent e) {
                }
            });
        shrinkSWTButton.addSelectionListener(
            new SelectionListener() {
                public void widgetSelected(SelectionEvent e) {
                     SwingUtilities.invokeLater(
                        new Runnable() {
                            public void run() {
                                rightSwingControl.growOrShrink(false);
                                leftSwingControl.growOrShrink(false);
                            }
                        });
                }
                public void widgetDefaultSelected(SelectionEvent e) {
                }
            });
        layoutSWTButton.addSelectionListener(
            new SelectionListener() {
                public void widgetSelected(SelectionEvent e) {
                    relayoutFromOutside();
                }
                public void widgetDefaultSelected(SelectionEvent e) {
                }
            });
    }

    void updateShrinkSWTButton() {
        assert EventQueue.isDispatchThread();    // On AWT event thread
        final boolean someShrinkable =
            rightSwingControl.isShrinkButtonEnabled()
            || leftSwingControl.isShrinkButtonEnabled();
        composite.getDisplay().asyncExec(
            new Runnable() {
                public void run() {
                    if (someShrinkable != shrinkSWTButton.isEnabled())
                        shrinkSWTButton.setEnabled(someShrinkable);
                }
            });
    }

    void someControlCreated() {
        assert EventQueue.isDispatchThread();    // On AWT event thread
        if (++numCreatedControls == 2) {
            // Done creating all controls.
            updateShrinkSWTButton();
        }
    }

    void relayoutFromOutside() {
        assert Display.getCurrent() != null;     // On SWT event thread

        composite.layout();
    }

    /**
     * Passing the focus request to the viewer's control.
     */
    public void setFocus() {
        rightSwingControl.setFocus();
    }

    public SwingControl[] getEmbeddedComposites() {
        return new SwingControl[] { leftSwingControl, rightSwingControl };
    }
}
