package org.eclipse.albireo.examples.plugin.views;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.albireo.core.ThreadingHandler;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.ui.part.ViewPart;

/**
 * This view allows to see the effects of resizing on SwingControl,
 * especially the flickering.
 * It shows four SwingControls in four quadrants. You can move the origin of
 * this coordinate system by clicking or by dragging the mouse pointer.
 */
public class TestResizeView extends ViewPart {

    public static String ID = "org.eclipse.albireo.examples.plugin.testResize"; //$NON-NLS-1$
    private SwingControl[][] swingControl;

    public void createPartControl(final Composite parent) {
        final Display display = Display.getCurrent();

        class MyListener implements MouseListener, MouseMotionListener {
            private void handleEvent(MouseEvent event) {
                //System.out.println(event);
                java.awt.Point p = new java.awt.Point(event.getX(), event.getY());
                SwingUtilities.convertPointToScreen(p, event.getComponent());
                java.awt.Point corner = new java.awt.Point(0, 0);
                SwingUtilities.convertPointToScreen(corner, swingControl[0][0].getSwingComponent());
                final int x = p.x - corner.x;
                final int y = p.y - corner.y;
                ThreadingHandler.getInstance().asyncExec(
                    display,
                    new Runnable() {
                        public void run() {
                            resizeAll(x, y);
                        }
                    });
            }
            // Implementation of MouseListener
            public void mouseClicked(MouseEvent event) {
                handleEvent(event);
            }
            public void mousePressed(MouseEvent event) {
                handleEvent(event);
            }
            public void mouseReleased(MouseEvent event) {
                handleEvent(event);
            }
            public void mouseEntered(MouseEvent event) {
                if (event.getButton() != MouseEvent.NOBUTTON)
                    handleEvent(event);
            }
            public void mouseExited(MouseEvent event) {
                if (event.getButton() != MouseEvent.NOBUTTON)
                    handleEvent(event);
            }
            // Implementation of MouseMotionListener
            public void mouseDragged(MouseEvent event) {
                handleEvent(event);
            }
            public void mouseMoved(MouseEvent event) {
            }
        }
        final MyListener listener = new MyListener();

        swingControl = new SwingControl[2][];
        for (int i = 0; i < 2; i++) {
            swingControl[i] = new SwingControl[2];
            for (int j = 0; j < 2; j++) {
                swingControl[i][j] =
                    new SwingControl(parent, SWT.NONE) {
                        protected JComponent createSwingComponent() {
                            return createQuadrant();
                        }
                        protected void afterComponentCreatedAWTThread() {
                            JComponent c = getSwingComponent();
                            c.addMouseListener(listener);
                            c.addMouseMotionListener(listener);
                        }
                        public Composite getLayoutAncestor() {
                            return parent;
                        }
                    };
            }
        }
        resizeAll(parent.getSize().x/2, parent.getSize().y/2);

        parent.setLayout(
            new Layout() {
                protected Point computeSize(Composite composite,
                                            int widthHint, int heightHint,
                                            boolean flushCache) {
                    int width = (widthHint > 0 ? widthHint : 200);
                    int height = (heightHint > 0 ? heightHint : 200);
                    return new Point(width, height);
                }
                protected void layout(Composite composite, boolean flushCache) {
                    int entireWidth = composite.getSize().x;
                    int entireHeight = composite.getSize().y;
                    int x = swingControl[0][0].getSize().x;
                    int y = swingControl[0][0].getSize().y;
                    if (x <= 0)
                        x = 1;
                    else if (x >= entireWidth)
                        x = entireWidth-1;
                    if (y <= 0)
                        y = 1;
                    else if (y >= entireHeight)
                        x = entireHeight-1;
                    resizeAll(x, y);
                }
            });
    }

    static JComponent createQuadrant() {
        JPanel comp =
            new JPanel() {
                public void paintComponent(Graphics g) {
                    int width = getWidth();
                    int height = getHeight();
                    for (int i = 0; 2*i < width && 2*i < height; i++) {
                        g.setColor(computePenColor(i));
                        g.drawRect(i, i, width-1-2*i, height-1-2*i);
                        if (2*i+1 < width && 2*i+1 < height)
                          g.drawRect(i+1, i+1, width-2-2*i, height-2-2*i);
                    }
                }
                private Color computePenColor(int i) {
                    float r = (float) Math.exp(-0.3 * ((4*i*i*i+2*i*i+2*i+3) % 7));
                    float g = (float) Math.exp(-0.2 * ((i*i*i+5*i*i+6*i+1) % 11));
                    float b = (float) Math.exp(-0.2 * ((3*i*i*i+3*i*i+i+3) % 13));
                    return new Color(r, g, b);
                }
            };
        return comp;
    }

    private void resizeAll(int x, int y) {
        //System.out.println("("+x+","+y+")");
        Control entireControl = swingControl[1][1].getParent();
        int entireWidth = entireControl.getSize().x;
        int entireHeight = entireControl.getSize().y;
        swingControl[0][0].setBounds(0, 0, x, y);
        swingControl[0][1].setBounds(x, 0, entireWidth-x, y);
        swingControl[1][0].setBounds(0, y, x, entireHeight-y);
        swingControl[1][1].setBounds(x, y, entireWidth-x, entireHeight-y);
    }

    public void setFocus() {
        swingControl[1][1].setFocus();
    }
}
